from __future__ import print_function, unicode_literals

import json
from os import listdir
from os.path import join, dirname, realpath, exists, isdir, relpath
import sys
from collections import defaultdict
from escher import __schema_version__
from escher.urls import root_directory

def index(directory):
    def list_cached(directory, kind):
        l = []
        if isdir(directory):
            for f in listdir(directory):
                tf = join(directory, f)
                if isdir(tf):
                    for f2 in listdir(tf):
                        if f2.endswith('.json'):
                            f2 = f2.replace('.json', '')
                            l.append({ 'organism': f,
                                       kind + '_name': f2 })
        return l
    return { 'maps': list_cached(join(directory, 'maps'), 'map'),
             'models': list_cached(join(directory, 'models'), 'model') }

if __name__=='__main__':
    directory = join(root_directory, __schema_version__)
    if not exists(directory):
        print('No directory to index')
        sys.exit()
    print('Indexing %s' % directory)
    index = index(directory)
    with open(join(directory, 'index.json'), 'w') as f:
        json.dump(index, f)
    print('Saved index.json')



# -*- coding: utf-8 -*-

from __future__ import print_function, unicode_literals

from escher import __schema_version__
from escher.urls import root_directory
from os.path import join
import re
import json

def validate_map(map_data):
    """Validate a map using the jsonschema, and some extra checks for consistency."""
    import jsonschema
    schema = get_jsonschema()
    jsonschema.validate(map_data, schema)

    # check that all segments have nodes, segments never connect midmarkers with
    # metabolites, and check that every metabolite is represented with
    # stoichiometry information
    (bad_segments,
     missing_multimarkers,
     missing_stoich,
     missing_gene_names) = check_map(map_data)
     
    error = ''
    if len(bad_segments) > 0:
        error += 'No nodes for segments: %s\n' % (', '.join(str(x) for x in bad_segments))
    # if len(missing_multimarkers) > 0:
    #     error += 'Segments connect midmarkers to metabolites: %s\n' % (', '.join(str(x) for x in missing_multimarkers))
    if len(missing_stoich) > 0:
        error += 'No non-zero stoichiometry for a connected metabolite node: %s\n' % (', '.join(str(x) for x in missing_stoich))
    if len(missing_gene_names) > 0:
        error += 'No gene name for gene in gene_reaction_rule: %s\n' % (', '.join(str(x) for x in missing_gene_names))
    if error != '':
        raise Exception(error)
    
def validate_schema():
    import jsonschema
    schema = get_jsonschema()
    jsonschema.Draft4Validator.check_schema(schema)    

def check_map(map_data):
    """Check reactions and metabolites.

    1. Make sure that nodes exist on either end of each segment

    2. check that every metabolite is represented with stoichiometry information

    3. that every gene in the gene_reaction_rule has a name

    """
    reactions = map_data[1]['reactions'];
    nodes = map_data[1]['nodes'];
    bad_segments = []
    missing_multimarkers = []
    missing_stoich = []
    missing_gene_names = []
    for _, reaction in reactions.items():
        metabolites = reaction['metabolites']
        for segment_id, segment in reaction['segments'].items():
            for n in ['to_node_id', 'from_node_id']:
                # check that the node exists
                if segment[n] not in nodes:
                    bad_segments.append((n, segment_id))
                else:
                    # check that the coefficients exist and are non-zero
                    node = nodes[segment[n]]
                    if node['node_type'] == 'metabolite':
                        if not any((node['bigg_id'] == m['bigg_id'] and abs(m['coefficient']) > 0) for m in metabolites):
                            missing_stoich.append((n, segment_id))

        # check gene reaction rule
        found_genes = genes_for_gene_reaction_rule(reaction['gene_reaction_rule'])
        for found_gene in found_genes:
            if not any((found_gene == g['bigg_id'] and 'name' in g) for g in reaction['genes']):
                missing_gene_names.append(found_gene)
    return bad_segments, missing_multimarkers, missing_stoich, missing_gene_names

def get_jsonschema():
    """Get the local jsonschema.

    """
    with open(join(root_directory, 'escher', 'jsonschema', __schema_version__), 'r') as f:
        return json.load(f)

def genes_for_gene_reaction_rule(rule):
    """ Find genes in gene_reaction_rule string.

    Arguments
    ---------

    rule: A boolean string containing gene names, parentheses, AND's and
    OR's.

    """
    
    # remove ANDs and ORs, surrounded by space or parentheses     
    rule = re.sub(r'([()\s])(?:and|or)([)(\s])', r'\1\2', rule)
    # remove parentheses
    rule = re.sub(r'\(|\)', r'', rule)
    # split on whitespace
    genes = [x for x in rule.split(' ') if x != '']
    return genes

if __name__=="__main__":
    from sys import argv
    if len(argv) < 2:
        raise Exception('Must supply a map JSON file')

    with open(argv[1], 'r') as f:
        map_data = json.load(f)
    validate_map(map_data)
    print('Your map passed inspection and is free of infection.')



from __future__ import print_function, unicode_literals

from escher.plots import Builder
from escher.urls import get_url
from escher.version import __version__, __schema_version__
from escher.urls import root_directory

from os.path import join, dirname, realpath
from jinja2 import Environment, PackageLoader

# set up jinja2 template location
env = Environment(loader=PackageLoader('escher', 'templates'))

def generate_static_site():
    print('Generating static site at %s' % root_directory)

    # index file
    template = env.get_template('index.html')
    
    data = template.render(d3=get_url('d3', 'local'),
                           boot_css=get_url('boot_css', 'local'),
                           index_css=get_url('index_css', 'local'),
                           favicon=get_url('favicon', 'local'),
                           logo=get_url('logo', 'local'),
                           documentation=get_url('documentation', protocol='https'),
                           github=get_url('github', protocol='https'),
                           index_js=get_url('index_js', 'local'),
                           index_gh_pages_js=get_url('index_gh_pages_js', 'local'),
                           version=__version__,
                           map_download=get_url('map_download', 'local'),
                           web_version=True,
                           server_index_url=get_url('server_index', 'local'),
                           # server_index=
                           # local_index=
                           can_dev=False,
                           # can_dev_js=
                           )
    
    with open(join(root_directory, 'index.html'), 'w') as f:
        f.write(data.encode('utf-8'))

    # viewer and builder
    for kind in ['viewer', 'builder']:
        for minified_js in [True, False]:
            js_source = 'local'
            enable_editing = (kind=='builder')

            # make the builder
            with open(join(root_directory, get_url('builder_embed_css', 'local')), 'r') as f:
                css = f.read()
            builder = Builder(embedded_css=css, safe=True, id='static')
            
            filepath = join(root_directory,
                            '%s%s.html' % (kind, '' if minified_js else '_not_minified'))
            with open(join(root_directory, get_url('server_index', source='local')), 'r') as f:
                index_json = f.read()
            html = builder.save_html(filepath=filepath,
                                     js_source=js_source,
                                     minified_js=minified_js,
                                     enable_editing=enable_editing,
                                     static_site_index_json=index_json)
    
if __name__=='__main__':
    generate_static_site()



from escher.version import __version__, __schema_version__

from escher.plots import (Builder, get_cache_dir, clear_cache, list_cached_maps,
                          list_cached_models, list_available_maps,
                          list_available_models)




def get_full_version(main_version, post_version=None):
    """Generate a PEP440 compliant version with an optional post-release."""
    if post_version is None:
        return main_version
    else:
        return '%s.post%s' % (main_version, post_version)

# software version
__version__ = '1.1.2'
# post-release version (required by PyPI & PEP440)
__post_version__ = '2'
# full version with post-release
__full_version__ = get_full_version(__version__, __post_version__)

# Escher map schema version
__schema_version__ = '1-0-0'



# -*- coding: utf-8 -*-

import sys
from sys import argv
from subprocess import call
import threading
import webbrowser
import os
from shutil import copy, move, rmtree
from os.path import join, dirname, realpath, exists
from glob import glob
import re

try:
    from setuptools import setup, Command
    from setuptools.command.sdist import sdist as SDistCommand
    from setuptools.command.bdist import bdist as BDistCommand
    from setuptools.command.upload import upload as UploadCommand
except ImportError:
    from distutils.core import setup, Command

directory = dirname(realpath(__file__))
sys.path.insert(0, join(directory, 'escher'))
version = __import__('version').__version__
full_version = __import__('version').__full_version__
escher = 'escher-%s.js' % version
escher_min = 'escher-%s.min.js' % version
builder_css = 'builder-%s.css' % version
builder_embed_css = 'builder-embed-%s.css' % version
port = 8789

class CleanCommand(Command):
    description = "Custom clean command that removes static site"
    user_options = []
    def initialize_options(self):
        pass
    def finalize_options(self):
        pass
    def run(self):
        def remove_if(x):
            if exists(x): rmtree(x)
        remove_if(join(directory, 'build'))
        remove_if(join(directory, 'dist'))
        # remove site files
        for f in glob(join(directory, '*.html')):
            os.remove(f)
        for f in glob(join(directory, 'builder-*.css')):
            os.remove(f)
        for f in glob(join(directory, 'escher-*.js')):
            os.remove(f)
        print('done cleaning')

class JSBuildCommand(Command):
    description = "Custom build command that generates escher lib files"
    user_options = []
    def initialize_options(self):
        pass
    def finalize_options(self):
        pass
    def run(self):
        call(['node',
              join(directory, 'bin', 'r.js'),
              '-o', join('escher', 'js', 'build', 'build.js'),
              'out='+join('escher', 'lib', escher),
              'optimize=none'])
        call(['node',
              join(directory, 'bin', 'r.js'),
              '-o', join('escher', 'js', 'build', 'build.js'),
              'out='+join('escher', 'lib', escher_min),
              'optimize=uglify'])
        print('done building js')
        
class BuildGHPagesCommand(Command):
    description = "Custom build command that generates static site, and copies escher libs"
    user_options = []
    def initialize_options(self):
        pass
    def finalize_options(self):
        pass
    def run(self):
        # copy files to top level
        copy(join('escher', 'lib', escher), '.')
        copy(join('escher', 'lib', escher_min), '.')
        copy(join('escher', 'css', builder_css), '.')
        copy(join('escher', 'css', builder_embed_css), '.')
        copy(join('escher', 'lib', escher), 'escher-latest.js')
        copy(join('escher', 'lib', escher_min), 'escher-latest.min.js')
        copy(join('escher', 'css', builder_css), 'builder-latest.css')
        copy(join('escher', 'css', builder_embed_css), 'builder-embed-latest.css')
        # generate the static site
        call(['python', join('escher', 'generate_index.py')])
        call(['python', join('escher', 'static_site.py')])
        print('Done building gh-pages')

class BuildRelease(Command):
    description = "Make file modifications for a new release version"
    user_options = [('version=', 'v', 'The new version')]
    def initialize_options(self):
        self.version = None
    def finalize_options(self):
        if self.version is None:
            print('Usage: python setup.py build_release --version=[version_number]')
            sys.exit()
    def run(self):
        old_version = version
        new_version = self.version
        # change the version
        file = join('escher', 'version.py')
        with open(file, 'r') as f:
            lines = f.readlines()
        with open(file, 'w') as f:
            for line in lines:
                f.write(line.replace(old_version, new_version))
        # update the specs
        file = join('spec', 'javascripts', 'support', 'jasmine.yml')
        with open(file, 'r') as f:
            lines = f.readlines()
        with open(file, 'w') as f:
            for line in lines:
                f.write(line.replace('escher-%s.js' % old_version, 'escher-%s.js' % new_version)) 
        # update the docs
        file = join('docs', 'conf.py')
        with open(file, 'r') as f:
            lines = f.readlines()
        with open(file, 'w') as f:
            for line in lines:
                f.write(line
                        .replace("version = '%s'" % old_version, "version = '%s'" % new_version)
                        .replace("release = '%s'" % old_version, "release = '%s'" % new_version))
        # move the files
        move(join('escher', 'lib', 'escher-%s.js' % old_version),
             join('escher', 'lib', 'escher-%s.js' % new_version))
        move(join('escher', 'lib', 'escher-%s.min.js' % old_version),
             join('escher', 'lib', 'escher-%s.min.js' % new_version))
        move(join('escher', 'css', 'builder-%s.css' % old_version),
             join('escher', 'css', 'builder-%s.css' % new_version))
        move(join('escher', 'css', 'builder-embed-%s.css' % old_version),
             join('escher', 'css', 'builder-embed-%s.css' % new_version))
        print('Done building release %s' % new_version)

class BuildDocs(Command):
    description = "Build the sphinx documentation"
    user_options = []
    def initialize_options(self):
        pass
    def finalize_options(self):
        pass
    def run(self):
        import platform
        if platform.system() == 'Windows':
            call('cd docs & make.bat html & cd ..', shell=True)
        else:
            call(['make', 'html', '-C', 'docs'])

class TestCommand(Command):
    description = "Custom test command that runs pytest and jasmine"
    user_options = [('jsonly', None, 'Only run jasmine tests'),
                    ('pyonly', None, 'Only run pytest'),
                    ('noweb', None, 'Skip run tests that require the Escher website')]
    def initialize_options(self):
        self.jsonly = False
        self.pyonly = False
        self.noweb = False
    def finalize_options(self):
        pass
    def run(self):
        if not self.jsonly:
            import pytest
            if self.noweb:
                exit_code = pytest.main(['-m', 'not web'])
            else:
                exit_code = pytest.main([])
        else:
            exit_code = 0
        if not self.pyonly:
            call(['jasmine', '--port=%d' % port])
        sys.exit(exit_code)

setup(name='Escher',
      version=full_version,
      author='Zachary King',
      url='https://escher.github.io',
      license='MIT',
      classifiers=[
          'Development Status :: 5 - Production/Stable',
          'License :: OSI Approved :: MIT License',
          'Topic :: Scientific/Engineering',
          'Topic :: Scientific/Engineering :: Visualization',
          'Programming Language :: Python :: 2',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.3',
          'Programming Language :: Python :: 3.4'
      ],
      keywords='visualization, pathway maps, web application, D3.js',
      packages=['escher'],
      package_data={'escher': ['css/web/*.css', 'css/*.css', 'templates/*',
                               'example_data/*', 'lib/*.js', 'lib/*.css',
                               'fonts/*', 'js/src/*.js', 'js/web/*.js',
                               'resources/*', 'jsonschema/*']},
      install_requires=['Jinja2>=2.7.3',
                        'tornado>=4.0.2',
                        'pytest>=2.6.2',
                        'jsonschema>=2.4.0'],
      extras_require={'docs': ['sphinx>=1.2',
                               'sphinx-rtd-theme>=0.1.6'],
                      'all': ['sphinx>=1.2',
                              'sphinx-rtd-theme>=0.1.6',
                              'jasmine>=2.2.0',
                              'ipython>=2.3.1',
                              'cobra>=0.3.0b4',
                              'wheel>=0.24.0',
                              'twine>=1.5.0'] },
      cmdclass={'clean': CleanCommand,
                'buildjs': JSBuildCommand,
                'buildgh': BuildGHPagesCommand,
                'build_release': BuildRelease,
                'build_docs': BuildDocs,
                'test': TestCommand})
